//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

// Own include
#include <Aux_SharedObject.h>

//! Default constructor.
Aux_SharedObject::Aux_SharedObject() : m_iRefCount(0)
{
}

//! Destructor.
Aux_SharedObject::~Aux_SharedObject()
{
}

//! Increments reference counter.
void Aux_SharedObject::IncRef()
{
#if defined(WIN32) || defined(_WIN32)
  InterlockedIncrement(&m_iRefCount);
#else
  static tbb::spin_mutex MUTEX;
  MUTEX.lock();
  ++m_iRefCount;
  MUTEX.unlock();
#endif
}

//! Decrements reference counter.
void Aux_SharedObject::DecRef()
{
#if defined(WIN32) || defined(_WIN32)
  LONG aRefCount = (LONG) m_iRefCount;
  m_iRefCount = (int) InterlockedDecrement(&aRefCount);
#else
  static tbb::spin_mutex MUTEX;
  MUTEX.lock();
  --m_iRefCount;
  MUTEX.unlock();
#endif
}

//! Returns the current number of references to the object.
//! \return number of references.
int Aux_SharedObject::NbRefs() const
{
  return m_iRefCount;
}

//! Dumps this object to the passed string stream.
//! \param stream [in/out] target stream.
void Aux_SharedObject::Dump(std::stringstream& stream) const
{
  stream << "Aux_SharedObject";
}
