//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

#ifndef Aux_SharedObject_HeaderFile
#define Aux_SharedObject_HeaderFile

// Windows includes
#include <windows.h>

// STD includes
#include <sstream>

//! Base class for all objects manipulated by smart pointers. These smart
//! pointers are intrusive, meaning that in order to have your own object
//! being controlled via the smart pointer, you need to sub-class
//! this Aux_SharedObject class. All such objects must be allocated in a heap
//! memory.
//!
//! The entire mechanism is based on reference counting. Each object has its
//! own counter incremented each time a new smart pointer appears, and decremented
//! once a smart pointer dies. Eventually, when the last smart pointer goes
//! out of scope (the smart pointers must be allocated in the automatic memory)
//! and reference counter gets the value of zero, the object is deleted.
//!
//! Note that this mechanism has no protection against cyclic dependencies.
class Aux_SharedObject
{
// Construction & destruction:
public:

  Aux_SharedObject();
  virtual ~Aux_SharedObject();

// Reference counting:
public:

  void IncRef();
  void DecRef();
  int  NbRefs() const;

public:

  virtual void
    Dump(std::stringstream& stream) const;

private:

  volatile LONG m_iRefCount; //!< Reference counter for smart pointers.

};

#endif
