//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

#ifndef BSpl_BasisEffectiveNDers_HeaderFile
#define BSpl_BasisEffectiveNDers_HeaderFile

//! Algorithm calculating those B-spline basis functions and their derivatives
//! which are non-vanishing in the given span. The target span is referenced by
//! its zero-based index. In order to use this functionality properly,
//! one should locate the target span by BasisFindSpan() and then ask this
//! routine to evaluate the basis functions (and derivatives) for the passed
//! parameter u.
//!
//! The derivatives are calculated according to the non-recurrent formula:
//!
//! <pre>
//! N^(k)_i,p = p! / (p - k)! Sum_{j=0,k} ( a_k,j N_i+j,p-k )
//! </pre>
//!
//! This algorithm is actually the implementation of A2.3 from
//! "The NURBS Book" by L. Piegl, W. Tiller.
class BSpl_BasisEffectiveNDers
{
// Construction & destruction:
public:

  BSpl_BasisEffectiveNDers();
  virtual ~BSpl_BasisEffectiveNDers();

public:

  void operator()(const double  u,
                  const double* pU,
                  const int     p,
                  const int     i,
                  const int     n,
                  double**      ders) const;

};

#endif
