//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

// BSpl includes
#include <BSpl_BasisFindSpan.h>

//! Constructor.
BSpl_BasisFindSpan::BSpl_BasisFindSpan()
{
}

//! Destructor.
BSpl_BasisFindSpan::~BSpl_BasisFindSpan()
{
}

//! Finds the target span index by binary search.
//! \param u  [in] target parameter.
//! \param pU [in] knot vector.
//! \param nU [in] number of knots.
//! \param p  [in] degree.
//! \return span index.
int BSpl_BasisFindSpan::operator()(const double  u,
                                   const double* pU,
                                   const int     nU,
                                   const int     p) const
{
  const int m = nU - 1;
  const int n = m - p - 1;
  if ( u == pU[n + 1] )
    return n;

  int min_idx = 0;
  int max_idx = nU;
  int mid_idx;
  bool isFound = false;

  do {
    mid_idx = (min_idx + max_idx) / 2;

    if ( mid_idx == min_idx || mid_idx == max_idx )
    {
      isFound = true;
      break;
    }

    double mid_u = pU[mid_idx];

    if ( mid_u <= u )
      min_idx = mid_idx;
    else if ( mid_u > u )
      max_idx = mid_idx;
  }
  while ( !isFound );

  return mid_idx;
}
