//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

// Own include
#include <BSpl_BasisN.h>

// BSpl includes
#include <BSpl_BasisFindSpan.h>

// STD includes
#include <math.h>

//! Constructor.
BSpl_BasisN::BSpl_BasisN()
{
}

//! Destructor.
BSpl_BasisN::~BSpl_BasisN()
{
}

double BSpl_BasisN::operator()(const double  u,
                               const double* pU,
                               const int     nU,
                               const int     p,
                               const int     i) const
{
  BSpl_BasisFindSpan FindSpan;
  const int span_idx = FindSpan(u, pU, nU, p);
  const int m = nU - 1;

  // Recurrent Cox-deBoor formulation start with 1-level step function
  if ( p == 0 )
  {
    double val = ( (span_idx == i) ? 1.0 : 0.0 );
    return val;
  }

  // Special cases
  if ( (i == 0 && u == pU[0]) || (i == m-p-1 && u == pU[m]) )
  {
    double val = 1.0;
    return val;
  }

  BSpl_BasisN N;
  double N_pprev_icurr = N(u, pU, nU, p - 1, i);
  double N_pprev_inext = N(u, pU, nU, p - 1, i + 1);
  double deltaU1 = (pU[i + p] - pU[i]);
  double deltaU2 = (pU[i + p + 1] - pU[i + 1]);

  const double prec = 1.0e-10;
  double N1, N2;

  if ( fabs(N_pprev_icurr) < prec && fabs(deltaU1) < prec )
    N1 = 0.0;
  else
    N1 = N_pprev_icurr * (u - pU[i]) / (pU[i + p] - pU[i]);

  if ( fabs(N_pprev_inext) < prec && fabs(deltaU2) < prec )
    N2 = 0.0;
  else
  {
    const double urange1 = pU[i + p + 1] - u;
    const double urange2 = pU[i + p + 1] - pU[i + 1];
    if ( fabs(urange1) < prec && fabs(urange2) < prec )
      N2 = 0.0;
    else
      N2 = N_pprev_inext * (pU[i + p + 1] - u) / (pU[i + p + 1] - pU[i + 1]);
  }

  return N1 + N2;
}
