//-----------------------------------------------------------------------------
// Created on: 12 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

#ifndef Extrema_DistWithPenalty_HeaderFile
#define Extrema_DistWithPenalty_HeaderFile

// Opt includes
#include <Opt_BSplineSplash.h>
#include <Opt_Function.h>

// Prim includes
#include <Prim_BSplineCurve.h>
#include <Prim_UV.h>

//! Distance function with penalty for niching optimization schemes.
class Extrema_DistWithPenalty : public Opt_Function<uv>
{
public:

  //! Constructor accepting curves to calculate the distance between.
  //! \param C1 [in] first curve.
  //! \param C2 [in] second curve.
  Extrema_DistWithPenalty(const smptr<bcurve>& C1,
                          const smptr<bcurve>& C2) : m_C1(C1), m_C2(C2)
  {}

public:

  //! Evaluated distance with penalties.
  //! \param x [in] point to evaluate the function in.
  //! \return function value.
  inline virtual double Value(const uv& x)
  {
    xyz P1, P2;
    m_C1->Eval(x.U(), P1);
    m_C2->Eval(x.V(), P2);

    double val = (P1 - P2).Dot(P1 - P2);
    for ( size_t s = 0; s < m_splashes.size(); ++s )
      val += m_splashes[s]->Value(x);

    return val;
  }

  //! Adds penalty "splash" to the objective function.
  //! \param splash [in] penalty splash to add.
  inline void AddPenalty(const smptr<Opt_Splash_NM>& splash)
  {
    m_splashes.push_back(splash);
  }

protected:

  smptr<bcurve>                       m_C1, m_C2; //!< Working curves.
  std::vector< smptr<Opt_Splash_NM> > m_splashes; //!< Penalties.

};

#endif
