//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

#ifndef Prim_BSplineCurve_HeaderFile
#define Prim_BSplineCurve_HeaderFile

// Prim includes
#include <Prim_Curve.h>
#include <Prim_Cx.h>

// STD includes
#include <vector>

//! B-spline curve in 3D. The B-spline degree, number of knots and number of
//! control points are bound with formula (m = n + p + 1), where m is the
//! index of the last knot, p is degree and n is the index of the last control
//! point.
class Prim_BSplineCurve : public Prim_Curve
{
// Construction & destruction:
public:

  Prim_BSplineCurve(const std::vector<xyz>& Poles,
                    const double*           pU,
                    const int               nU,
                    const int               p);

  Prim_BSplineCurve(const std::vector<Prim_XYZ>& Poles,
                    const std::vector<double>&   U,
                    const int                    p);

  virtual ~Prim_BSplineCurve();

public:

  virtual double
    MinParameter() const;

  virtual double
    MaxParameter() const;

  virtual void
    Eval(const double u,
         xyz& P) const;

  virtual void
    Eval_Dk(const double u,
            const int k,
            xyz& dkC) const;

  virtual void
    Eval_Dk(double** dN,
            const double u,
            const int k,
            xyz& dkC) const;

  virtual Prim_Cx
    Continuity() const;

public:

  //! Accessor for the collection of poles.
  //! \return poles of B-spline curve.
  inline const std::vector<Prim_XYZ>& Poles() const
  {
    return m_poles;
  }

  //! Accessor for array of knots.
  //! \return array of knots repacked into vector.
  inline std::vector<double> Knots() const
  {
    std::vector<double> res;
    for ( int k = 0; k < m_iU; ++k )
      res.push_back(m_pU[k]);

    return res;
  }

  //! Returns the number of knots.
  //! \return number of knots.
  inline int NumOfKnots() const
  {
    return m_iU;
  }

  //! Returns the degree of the curve.
  //! \return degree.
  inline int Degree() const
  {
    return m_iDeg;
  }

private:

  void init(const std::vector<Prim_XYZ>& Poles,
            const double* U,
            const int nU,
            const int p);

private:

  //! Poles of B-spline curve.
  std::vector<Prim_XYZ> m_poles;

  //! Knot vector.
  double* m_pU;

  //! Number of knots.
  int m_iU;

  //! Degree.
  int m_iDeg;

};

//! Convenience shortcut.
typedef Prim_BSplineCurve bcurve;

#endif
