//-----------------------------------------------------------------------------
// Created on: 13 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

// Own include
#include <Prim_UV.h>

//! Default constructor. Initializes point coordinates with origin values:
//! (0, 0).
Prim_UV::Prim_UV()
{
  m_fU = m_fV = 0.0;
}

//! Complete constructor.
//! \param u [in] first coordinate.
//! \param v [in] second coordinate.
Prim_UV::Prim_UV(const double u, const double v)
{
  m_fU = u;
  m_fV = v;
}

//! Copy constructor.
//! \param UV [in] point to copy.
Prim_UV::Prim_UV(const Prim_UV& UV)
{
  this->operator=(UV);
}

//! Destructor.
Prim_UV::~Prim_UV()
{
}

//! Returns modulus of the point's radius vector.
//! \return modulus.
double Prim_UV::Modulus() const
{
  return sqrt( this->SquaredModulus() );
}

//! Returns squared modulus of the point's radius vector.
//! \return modulus.
double Prim_UV::SquaredModulus() const
{
  return m_fU*m_fU + m_fV*m_fV;
}

//! Calculates dot product between this and another vector.
//! \param UV [in] another vector.
//! \return dot product.
double Prim_UV::Dot(const Prim_UV& UV) const
{
  return m_fU*UV.m_fU + m_fV*UV.m_fV;
}

//! Assignment operator.
//! \param UV [in] point to copy into this one.
//! \return this one.
Prim_UV& Prim_UV::operator=(const Prim_UV& UV)
{
  m_fU = UV.m_fU;
  m_fV = UV.m_fV;
  return *this;
}

//! Multiplies copy of point by the passed scalar value.
//! \param coeff [in] scalar value to multiply point by.
//! \return resulting point.
Prim_UV Prim_UV::operator*(const double coeff) const
{
  Prim_UV UV_Copy(*this);
  UV_Copy *= coeff;
  return UV_Copy;
}

//! Multiplies this point by the passed scalar value.
//! \param coeff [in] scalar value to multiply point by.
//! \return this point multiplied by the passed scalar.
Prim_UV Prim_UV::operator*=(const double coeff)
{
  this->m_fU *= coeff;
  this->m_fV *= coeff;
  return *this;
}

//! Divides copy of point by the passed scalar value.
//! \param coeff [in] scalar value to divide point by.
//! \return resulting point.
Prim_UV Prim_UV::operator/(const double coeff) const
{
  Prim_UV UV_Copy(*this);
  UV_Copy /= coeff;
  return UV_Copy;
}

//! Divides this point by the passed scalar value.
//! \param coeff [in] scalar value to divide point by.
//! \return this point multiplied by the passed scalar.
Prim_UV Prim_UV::operator/=(const double coeff)
{
  this->m_fU /= coeff;
  this->m_fV /= coeff;
  return *this;
}

//! Adds the passed point to the copy of this one.
//! \param UV [in] point to add.
//! \return result of addition.
Prim_UV Prim_UV::operator+(const Prim_UV& UV) const
{
  Prim_UV UV_Copy(*this);
  UV_Copy += UV;
  return UV_Copy;
}

//! Adds the passed point to this one.
//! \param UV [in] point to add.
//! \return result of addition.
Prim_UV& Prim_UV::operator+=(const Prim_UV& UV)
{
  this->m_fU += UV.m_fU;
  this->m_fV += UV.m_fV;
  return *this;
}

//! Inverts the copy of this point.
//! \return result of inversion.
Prim_UV Prim_UV::Invert() const
{
  Prim_UV UV_Copy(*this);
  UV_Copy.m_fU = -this->m_fU;
  UV_Copy.m_fV = -this->m_fV;
  return UV_Copy;
}

//! Subtracts the passed point from the copy of this one.
//! \param UV [in] point to subtract.
//! \return result of subtraction.
Prim_UV Prim_UV::operator-(const Prim_UV& UV) const
{
  return this->operator+( UV.Invert() );
}

//! Adds the passed point to this one.
//! \param UV [in] point to add.
//! \return result of subtraction.
Prim_UV& Prim_UV::operator-=(const Prim_UV& UV)
{
  return this->operator+=( UV.Invert() );
}
