//-----------------------------------------------------------------------------
// Created on: 12 October 2015
// Created by: Sergey SLYADNEV
// Web-site: http://quaoar.su/blog
//-----------------------------------------------------------------------------

#ifndef Prim_UV_HeaderFile
#define Prim_UV_HeaderFile

// STD includes
#include <float.h>
#include <math.h>
#include <stddef.h>

//! Geometric primitive for 2D point.
class Prim_UV
{
public:

  static int num_coordinates()
  {
    return 2;
  }

// Construction & destruction:
public:

  Prim_UV();
  Prim_UV(const double u, const double v);
  Prim_UV(const Prim_UV& UV);
  virtual ~Prim_UV();

public:

  //! Returns U coordinate of the 2D point.
  //! \return U coordinate.
  inline double U() const
  {
    return m_fU;
  }

  //! Sets U coordinate.
  //! \param u [in] value to set.
  inline void SetU(const double u)
  {
    m_fU = u;
  }

  //! Returns V coordinate of the 2D point.
  //! \return V coordinate.
  inline double V() const
  {
    return m_fV;
  }

  //! Sets V coordinate.
  //! \param v [in] value to set.
  inline void SetV(const double v)
  {
    m_fV = v;
  }

  //! Returns coordinate by its 0-based index.
  //! \param idx [in] 0 for U, 1 for V.
  //! \return requested coordinate.
  inline double Coord(const int idx) const
  {
    if ( idx == 0 )
      return this->U();

    if ( idx == 1 )
      return this->V();

    return DBL_MAX;
  }

  //! Updates coordinate having the specified 0-based index with the
  //! passed value.
  //! \param idx [in] 0 for U, 1 for V.
  //! \param val [in] value to set.
  inline double SetCoord(const int idx,
                         const double val)
  {
    double* coord = NULL;

    if ( idx == 0 )
      coord = &m_fU;
    if ( idx == 1 )
      coord = &m_fV;

    if ( coord )
      *coord = val;

    return DBL_MAX;
  }

public:

  double Modulus       ()                  const;
  double SquaredModulus()                  const;
  double Dot           (const Prim_UV& UV) const;

public:

  Prim_UV& operator=  (const Prim_UV& UV);
  Prim_UV  operator*  (const double coeff) const;
  Prim_UV  operator*= (const double coeff);
  Prim_UV  operator/  (const double coeff) const;
  Prim_UV  operator/= (const double coeff);
  Prim_UV  operator+  (const Prim_UV& UV)  const;
  Prim_UV& operator+= (const Prim_UV& UV);
  Prim_UV  Invert()                        const;
  Prim_UV  operator-  (const Prim_UV& UV)  const;
  Prim_UV& operator-= (const Prim_UV& UV);

private:

  double m_fU; //!< U coordinate.
  double m_fV; //!< V coordinate.

};

//! Convenience alias.
typedef Prim_UV uv;

#endif
